import string
from cStringIO import StringIO
import zamltext
from ZamlViewerBase import ZamlViewerBase
from OFS import ZDOM

class ZamlEditorBase(ZamlViewerBase):
    """Mixin class with functions useful for the ZAML editor.
    """
    def get_neighbor_element_node(self, node):
        """Get the next element node, if possible, otherwise get
        previous, if possible. If no other element nodes, return
        None.
        """
        next_node = node
        while 1:
            next_node = next_node.getNextSibling()
            if (next_node == None or
                next_node.getNodeType() == ZDOM.ELEMENT_NODE):
                break
        if next_node != None:
            return next_node
        next_node = node
        while 1:
            next_node = next_node.getPreviousSibling()
            if (next_node == None or
                next_node.getNodeType() == ZDOM.ELEMENT_NODE):
                return next_node
        return None
            
    def format_text(self, data, max_length=65):
        """Format text for textarea display.
        """
        lines = break_text(data, max_length)
        return string.join(lines, "\n")

    def set_text(self, node, data):
        """Add text with embedded <strong>, <em> and <link> nodes to node.
        """
        # get text split by tokens
        tokens = zamltext.zamltext(data)
        
        # remove old subnodes of node
        for subnode in node.getChildNodes():
            node.removeChild(subnode)
            
        doc = node.getOwnerDocument()
    
        # now add all tokens to the XML
        for token, text in tokens:
            if token:
                if token == zamltext.TOKEN_STRONG:
                    nodename = 'strong'
                elif token == zamltext.TOKEN_EM:
                    nodename = 'em'
                else:
                    raise Error, "Unknown token %s" % token
                newnode = doc.createElement(nodename)
                newnode.appendChild(doc.createTextNode(text))
            else:
                newnode = doc.createTextNode(text)
            node.appendChild(newnode)

    def get_text(self, node):
        """Render subnodes of node as text with *italic* and **bold**.
        """
        result = StringIO()
        for subnode in node.getChildNodes():
            nodename = subnode.getNodeName()
            if nodename == '#text':
                result.write(subnode.getNodeValue())
            elif nodename == 'em':
                result.write('*%s*' % subnode.text_content())
            elif nodename == 'strong':
                result.write("**%s**" % subnode.text_content())
            elif nodename == 'link':
                result.write('_%s|%s_' % (subnode.title,
                                          subnode.text_content()))
            else:
                raise Error, "Unknown subnode %s." % nodename
        return result.getvalue()
        
def break_text(data, max_length):
    """Break up text data into a set of lines.
    """
    # split the data into individual words
    words = string.split(data)

    # the lines we are going to fill
    lines = []

    line_length = 0
    line_words = []

    for word in words:
        # get length of word
        word_length = len(word)
        
        # add word to line
        line_words.append(word)
        # add length for space and the word to the line
        line_length = line_length + 1 + word_length
        
        # if line is too long
        if line_length >= max_length:
            if len(line_words) == 1:
                # line has just one very long word, add it
                lines.append(line_words[0])
                # new line without words yet, also eat room for first space
                line_length = -1
                line_words = []
            else:
                # use all the words in the line but the last one
                lines.append(string.join(line_words[:-1]))
                # new line with this word in it (no space yet)
                line_length = word_length
                line_words = [word]
                
    # add any remaining words in last line
    if len(line_words) > 0:
        lines.append(string.join(line_words))
    return lines

class Error(Exception):
    pass

def test_break():
    text = """
    this is normal text but it should be broken up
    ThisIsAVeryLongWordTooLongInFact
    ThisIsAlsoAVeryLongWordYeahYeah
    and this is also normal text which should be broken up
    into various lines, and it'll all work wonderfully, just you watch
    """
    
    for max_length in range(1, 80):
        for line in break_text(text, max_length):

            # if we encounter a line that's too long
            if len(line) >= max_length:
                # check if it's a single big word, if not, give error
                if len(string.split(line)) > 1:
                    raise Error, "Test failed, "\
                          "multi word line is longer than max_length."
            print line

if __name__ == "__main__":
    test()
    






