import string

TOKEN_STRONG = ('**', '**')
TOKEN_EM = ('*', '*')

TOKENS = [TOKEN_STRONG, TOKEN_EM]

def zamltext(data):
    """Converts data into a list of zaml node objects.
    """
    return split_tokens(data, TOKENS)

def next_marker(data, i, markers):
    """Starting at position i in data, find the next special marker.
    """
    min = len(data)
    found_marker = None
    for marker in markers:
        pos = string.find(data, marker, i)
        if pos != -1 and pos < min:
            min = pos
            found_marker = marker
    return min, found_marker

def split_markers(data, markers):
    """Split up text based on markers. Markers are in their own string.
    """
    l = len(data)
    i = 0
    result = []
    while i < l:
        j, marker = next_marker(data, i, markers)
        if j > i:
            result.append(data[i:j])
        if marker:
            i = j + len(marker)
            result.append(data[j:i])
        else:
            i = j
    return result

def split_tokens(data, tokens):
    """Split up text based on tokens.

    A token is a tuple containing of begin and end marker strings.
    Very simple; nested tokens aren't allowed.
    Raises ZamlTextError if data contains strange beginnings/endings.
    """
    markers = []
    # get all markers
    for begin_marker, end_marker in tokens:
        markers.append(begin_marker)
        markers.append(end_marker)
    # split text on marker basis
    data = split_markers(data, markers)

    # now get all tokens in a list of tuples (token, text)
    result = []
    l = len(data)
    i = 0
    while i < l:
        token = find_token(data[i], tokens)
        if not token:
            result.append((None, data[i]))
        else:
            try:
                # skip to text
                i = i + 1
                # add text
                result.append((token, data[i]))
                # skip to end marker
                i = i + 1
                # check end marker
                if data[i] != token[1]:
                    raise ZamlTextError, ("Found '%s' "
                                          "but expected end marker '%s'." %
                                          (data[i], token[1]))
            except IndexError:
                raise ZamlTextError, ("Start marker '%s' found, "
                                      "but token is incomplete." % (token[0]))
        # go to next element
        i = i + 1
    return result

class ZamlTextError(Exception):
    pass

def find_token(element, tokens):
    for token in tokens:
        if element == token[0]:
            return token
    return None

def test_split_tokens():
    tokens = [('**', '**'), ('*', '*')]
    print split_tokens('this is a lot of *text*', tokens)
    print split_tokens("this is a **test** with a few *markers thrown in*,"
                       "isn't it a lot of *fun*, heh, **heh**?", tokens)
    print split_tokens("*boundary condition*", tokens)
    print split_tokens("another boundary condition", tokens)

    tokens = [('[', ']'), ('^s^', '^e^')]
    print split_tokens("this is a [test]", tokens)
    print split_tokens("this is ^s^another^e^ test.", tokens)
    
    #print split_tokens("*messy input**", tokens)
    #print split_tokens("**more messy input*", tokens)
    #print split_tokens("*even**more**", tokens)
    
def test_split_markers():
    markers = ['**', '*']
    
    print split_markers("this is a **test** with a few *markers thrown in*,"
                        "isn't it lots of fun?", markers)

    print split_markers("*a boundary condition*", markers)
    print split_markers("another boundary condition", markers)
    print split_markers("***something***strange", markers)
    
def test():
    test_split_tokens()

if __name__ == "__main__":
    test()


